@@ -1,8 +1,10 @@
 class UnsupportedFormatException(Exception):
     """ An exception raised due to unsupported formats."""
-    pass
 
 
 class TerminalException(Exception):
     """ An exception raised due to unrecoverable failures."""
-    pass
+
+
+class NoFrameRateException(Exception):
+    """ An exception raised due to frame rate not found."""
